package logwire.web.bo.loader;

import logwire.core.bo.annotation.BizModel;
import logwire.core.bo.annotation.Column;
import logwire.core.bo.object.BizObject;
import logwire.core.exceptions.ApplicationException;
import logwire.core.meta.model.Model;
import logwire.core.meta.query.IQuery;
import logwire.core.resource.BeanLoader;
import logwire.core.resource.loader.JavaModelBeanLoader;
import logwire.core.tenant.TenantClassLoader;
import logwire.core.tenant.TenantProject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.io.DefaultResourceLoader;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;

public class BoModelBeanLoader extends BeanLoader<IQuery, TenantProject> implements JavaModelBeanLoader {

    @Autowired
    LoaderUtil loaderUtil;

    @Override
    public void load(TenantProject project, TenantProject input, Consumer<IQuery> consumer) throws Exception {
        TenantClassLoader classLoader = project.getTenantBeanContext().getTenantClassLoader();
        ClassPathScanningCandidateComponentProvider provider = loaderUtil.getScanningBeanProvider();
        provider.setResourceLoader(new DefaultResourceLoader(classLoader));
        provider.addIncludeFilter(loaderUtil.getAbstractTypeFilter(BizObject.class));
        Set<BeanDefinition> beans = provider.findCandidateComponents("logwire.web.bo");

        List<BeanDefinition> bizModelList = beans.stream().filter(beanDefinition -> {
            try {
                Class<?> clazz = classLoader.loadClass(beanDefinition.getBeanClassName());
                BizModel bizModelAnnotation = clazz.getAnnotation(BizModel.class);
                return bizModelAnnotation != null;
            } catch (ClassNotFoundException e) {
                throw new ApplicationException("");
            }
        }).collect(Collectors.toList());


        for (BeanDefinition beanDefinition : beans) {
            Class<?> clazz = classLoader.loadClass(beanDefinition.getBeanClassName());
            //判断是否独立model
            BizModel bizModelAnnotation = clazz.getAnnotation(BizModel.class);
            if (bizModelAnnotation != null) {
                //独立model
                Model model = new Model();
                model.setName(clazz.getSimpleName());
                model.setName(clazz.getSimpleName());
                model.setVerboseName(bizModelAnnotation.label());
                model.setIncludeAuditFields(true);
                model.setIncludeVersionField(true);
                model.setIncludeDomainField(true);
                initField(clazz, model);

            } else {
                //判断父类是否包含 @BizModel ，若包含则

            }


          /*  this.initModel(clazz, model);
            this.initFields(clazz, model);
            this.initIndex(clazz, model);
            this.initSign(model);
            consumer.accept(model);*/

        }


    }

    private void initField(Class<?> clazz, Model model) {
        Field[] fields = clazz.getDeclaredFields();
        for (Field f : fields) {
            Column column = f.getAnnotation(Column.class);
            if (column != null) {
            /*    model.*/
            }
        }
    }

    @Override
    public boolean accept(TenantProject input) {
        return true;
    }
}
